<?php
/**
 * @package     Wettermodul
 * @author      Thomas Hunziker <admin@bakual.net>
 * @copyright   (C) 2014 - Thomas Hunziker
 * @license     http://www.gnu.org/licenses/gpl.html
 **/

// DWD Wettervorhersage Modul
// Wetterdaten:(c) DWD - Deutscher Wetterdienst, Offenbach
// Grafiken: (c) J. Correa - www.jcorrea.es
// Modul: (c) M. Bollmann - www.stranddorf.de
// **************************************************************************
// Das Modul lädt aktuelle Wetterdaten und Vorhersagen vom FTP Server des DWD.
// Die Daten werden lokal zwischengespeichert und grafisch aufgearbeitet. 
// Die Wetterdaten der Grundversorgung dürfen frei verwendet werden, sind jedoch urheberrechtlich geschützt.
// Der Copyright-Vermerk und der Link dürfen nicht entfernt werden! Anpassungen an das eigene Webdesign sind natürlich gestattet.
// Der Quelltext ist zwar nicht schön, dafür in liebevoller Handarbeit gebastelt!
// **************************************************************************

defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * Helper class for DWD Wettermodul
 *
 * @since  1.0
 */
abstract class ModDwdwetterHelper
{
	public static function datenLaden($tag, $passwort, $user, $aktuell, $cachezeit)
	{
		$tag_cache = JPATH_SITE . '/modules/mod_dwd_wettermodul/' . $tag . '_cache.txt';

		if (file_exists($tag_cache))
		{
			$cache_zeit = (time() - filemtime($tag_cache));
		}

		if (isset($cache_zeit) && $cache_zeit < $cachezeit && file_exists($tag_cache))
		{
			$fp = @fopen($tag_cache, 'r');
			$wetterdaten = @fread($fp, filesize($tag_cache));
			@fclose($fp);
		}
		else
		{
			$fp = @fopen($tag_cache, 'w');
			@fclose($fp);
			@unlink($tag_cache);

			if ($aktuell==true)
			{
				$remote_file = '/gds/gds/specials/observations/tables/germany/';
				$dir = 'ftp://' . $user . ':' . $passwort . '@ftp-outgoing2.dwd.de' . $remote_file;
				$pattern = '/SXDL99_(.*)/';
				$dh = @opendir($dir);
				$files = array();

				while (false !== ($filename = @readdir($dh)))
				{
					if (preg_match($pattern, $filename))
					{
						$files[] = $filename;
					}
				} 

				@closedir($dh);
				@rsort($files);
				$latest = count($files) > 0 ? $files[0] : null;
				$dir .= $latest;
				$handle = @fopen ($dir, 'r');
				$ii = 0;

				while ($ii != 87)
				{
					$wetterdaten .= @fgets($handle, 8192);
					$ii++;
				}

				@fclose($handle); 
			}
			else
			{
				$remote_file = '/gds/gds/specials/forecasts/tables/germany/' . $tag;
				$handle = @fopen ('ftp://' . $user . ':' . $passwort . '@ftp-outgoing2.dwd.de' . $remote_file, 'r');
				$ii = 0;

				while ($ii != 24)
				{
					$wetterdaten .= @fgets($handle, 8192);
					$ii++;
				}

				@fclose($handle); 
			}

			if ($fp = fopen($tag_cache, 'w'))
			{
				fwrite($fp, $wetterdaten);
				fclose($fp);
			}
		}

		return $wetterdaten;
	}

	public static function vorhersageAuswerten($wetterdaten, $suchmuster)
	{
		preg_match($suchmuster, $wetterdaten, $treffer);
		$teile = str_word_count(trim($treffer[1]), 1, '-öäü1234567890.,');

		if ($teile[0] !== NULL)
		{
			$zerlegen['vorhersage']['temp'] = $teile[0] . ' &deg;C';
		}
		else
		{
			$zerlegen['vorhersage']['temp'] = '-- &deg;C';
		}

		$check = array('leichter', 'starker', 'kräftiger', 'vereinzelt', 'in', 'schweres', 'starkes');
		if (in_array($teile[1], $check))
		{
			$teile[1] = $teile[1] . ' ' . $teile[2];
		}

		if ($teile[1] == 'kein')
		{
			$teile[1] = 'heiter';
		}

		switch ($teile[1])
		{
			case 'heiter':
				$zerlegen['vorhersage']['himmel'] = 'heiter.png'; 
				break;
			case 'bewölkt':
				$zerlegen['vorhersage']['himmel'] = 'bewolkt.png'; 
				break;
			case 'bedeckt':
				$zerlegen['vorhersage']['himmel'] = 'bedeckt.png'; 
				break;
			case 'Sonne':
			case 'wolkenlos':
				$zerlegen['vorhersage']['himmel'] = 'sonne.png'; 
				break;
			case 'Regen':
			case 'Regenschauer':
			case 'leichter Regen':
			case 'leichter Schneeregen':
			case 'Schneeregen':
				$zerlegen['vorhersage']['himmel'] = 'leichtregen.png'; 
				break;
			case 'kräftiger Regen':
			case 'kräftiger Regenschauer':
				$zerlegen['vorhersage']['himmel'] = 'starkregen.png'; 
				break;
			case 'Glatteisbildung':
				$zerlegen['vorhersage']['himmel'] = 'schnee.png'; 
				break;
			case 'Schnee':
			case 'Schneefall':
			case 'leichter Schneefall':
			case 'starker Schneefall':
			case 'leichter Schneeregen':
			case 'Schneeregenschauer':
			case 'kräftiger Schneefall':
			case 'kräftiger Schneeregen':
			case 'kräftiger Schneeregenschauer':
			case 'kräftiger Schneeschauer':
			case 'Schneefegen':
			case 'Schneeschauer':
				$zerlegen['vorhersage']['himmel'] = 'schnee.png'; 
				break;
			case 'Graupelschauer':
			case 'Hagelschauer':
			case 'kräftiger Graupelschauer':
			case 'kräftiger Hagelschauer':
				$zerlegen['vorhersage']['himmel'] = 'hagel.png'; 
				break;
			case 'Gewitter':
			case 'schweres Gewitter':
			case 'starkes Gewitter':
				$zerlegen['vorhersage']['himmel'] = 'gewitter.png'; 
				break;
			case 'Nebel':
			case 'Dunst':
			case 'in Wolken':
			case 'Sandsturm':
				$zerlegen['vorhersage']['himmel'] = 'nebel.png'; 
				break;
			default:
				$zerlegen['vorhersage']['himmel'] = 'bewolkt.png'; 
				break;
		}
		$zerlegen['vorhersage']['beschreibung'] = htmlentities($teile[1]);

		return $zerlegen;
	}

	public static function aktuelleDaten($wetterdaten,$daten,$stunde)
	{
		switch ($daten)
		{
			case '1': $suchmuster = '/Nordseeboje II (.*)/'; break;
			case '2': $suchmuster = '/Helgoland (.*)/'; break;
			case '3': $suchmuster = '/List\/Sylt (.*)/'; break;
			case '4': $suchmuster = '/Schleswig (.*)/'; break;
			case '5': $suchmuster = '/Grosst. Fehmarnbelt (.*)/'; break;
			case '6': $suchmuster = '/Leuchtturm Kiel (.*)/'; break;
			case '7': $suchmuster = '/[^Leuchtturm ]Kiel (.*)/'; break;
			case '8': $suchmuster = '/Fehmarn (.*)/'; break;
			case '9': $suchmuster = '/Arkona (.*)/'; break;
			case '10': $suchmuster = '/Norderney (.*)/'; break;
			case '11': $suchmuster = '/Leuchtt. Alte Weser (.*)/'; break;
			case '12': $suchmuster = '/Cuxhaven (.*)/'; break;
			case '13': $suchmuster = '/Hamburg-Flh. (.*)/'; break;
			case '14': $suchmuster = '/Schwerin (.*)/'; break;
			case '15': $suchmuster = '/Rostock (.*)/'; break;
			case '16': $suchmuster = '/Greifswald (.*)/'; break;
			case '17': $suchmuster = '/Emden (.*)/'; break;
			case '18': $suchmuster = '/Bremen-Flh. (.*)/'; break;
			case '19': $suchmuster = '/Marnitz (.*)/'; break;
			case '20': $suchmuster = '/Neuruppin (.*)/'; break;
			case '21': $suchmuster = '/Angermünde (.*)/'; break;
			case '22': $suchmuster = '/Münster\/Osnabr.-Flh.(.*)/'; break;
			case '23': $suchmuster = '/Hannover-Flh. (.*)/'; break;
			case '24': $suchmuster = '/Magdeburg (.*)/'; break;
			case '25': $suchmuster = '/Potsdam (.*)/'; break;
			case '26': $suchmuster = '/Berlin-Tegel (.*)/'; break;
			case '27': $suchmuster = '/Berlin-Tempelhof (.*)/'; break;
			case '28': $suchmuster = '/Lindenberg (.*)/'; break;
			case '29': $suchmuster = '/Düsseldorf-Flh. (.*)/'; break;
			case '30': $suchmuster = '/Kahler Asten (.*)/'; break;
			case '31': $suchmuster = '/Bad Lippspringe (.*)/'; break;
			case '32': $suchmuster = '/Kassel (.*)/'; break;
			case '33': $suchmuster = '/Fritzlar (.*)/'; break;
			case '34': $suchmuster = '/Brocken (.*)/'; break;
			case '35': $suchmuster = '/Leipzig-Flh. (.*)/'; break;
			case '36': $suchmuster = '/Dresden-Flh. (.*)/'; break;
			case '37': $suchmuster = '/Cottbus (.*)/'; break;
			case '38': $suchmuster = '/Görlitz  (.*)/'; break;
			case '39': $suchmuster = '/Aachen-Orsbach (.*)/'; break;
			case '40': $suchmuster = '/Nürburg (.*)/'; break;
			case '41': $suchmuster = '/Köln\/Bonn-Flh. (.*)/'; break;
			case '42': $suchmuster = '/Gießen\/Wettenberg (.*)/'; break;
			case '43': $suchmuster = '/Wasserkuppe (.*)/'; break;
			case '44': $suchmuster = '/Meiningen (.*)/'; break;
			case '45': $suchmuster = '/Erfurt (.*)/'; break;
			case '46': $suchmuster = '/Gera (.*)/'; break;
			case '47': $suchmuster = '/Fichtelberg (.*)/'; break;
			case '48': $suchmuster = '/Trier  (.*)/'; break;
			case '49': $suchmuster = '/Hahn-Flh. (.*)/'; break;
			case '50': $suchmuster = '/Frankfurt/M-Flh. (.*)/'; break;
			case '51': $suchmuster = '/OF-Wetterpark  (.*)/'; break;
			case '52': $suchmuster = '/Würzburg (.*)/'; break;
			case '53': $suchmuster = '/Bamberg (.*)/'; break;
			case '54': $suchmuster = '/Hof (.*)/'; break;
			case '55': $suchmuster = '/Weiden (.*)/'; break;
			case '56': $suchmuster = '/Saarbrücken-Flh. (.*)/'; break;
			case '57': $suchmuster = '/Karlsruhe-Rheinst. (.*)/'; break;
			case '58': $suchmuster = '/Mannheim (.*)/'; break;
			case '59': $suchmuster = '/Stuttgart-Flh. (.*)/'; break;
			case '60': $suchmuster = '/Öhringen (.*)/'; break;
			case '61': $suchmuster = '/Nürnberg-Flh. (.*)/'; break;
			case '62': $suchmuster = '/Regensburg (.*)/'; break;
			case '63': $suchmuster = '/Straubing (.*)/'; break;
			case '64': $suchmuster = '/Grosser Arber (.*)/'; break;
			case '65': $suchmuster = '/Lahr (.*)/'; break;
			case '66': $suchmuster = '/Freudenstadt (.*)/'; break;
			case '67': $suchmuster = '/Stötten (.*)/'; break;
			case '68': $suchmuster = '/Augsburg (.*)/'; break;
			case '69': $suchmuster = '/München-Flh. (.*)/'; break;
			case '70': $suchmuster = '/Fürstenzell (.*)/'; break;
			case '71': $suchmuster = '/Feldberg\/Schw. (.*)/'; break;
			case '72': $suchmuster = '/Konstanz (.*)/'; break;
			case '73': $suchmuster = '/Kempten (.*)/'; break;
			case '74': $suchmuster = '/Oberstdorf (.*)/'; break;
			case '75': $suchmuster = '/Zugspitze (.*)/'; break;
			case '76': $suchmuster = '/Hohenpeissenberg (.*)/'; break;
			case '77': $suchmuster = '/Wendelstein (.*)/'; break;
			default:  $suchmuster = '/Fehmarn (.*)/'; break;
		}

		$gliederung = '/Station (.*)/';
		preg_match($gliederung, $wetterdaten, $legende);
		$glieder = str_word_count(trim($legende[1]), 1, '-öäü1234567890.,/');
		$k = 0;

		for($count = 1; $count < 21; $count++)
		{
			$position[$count] = 20;
		}

		foreach($glieder as $j)
		{
			switch ($j)
			{
				case 'Höhe': $position[0] = $k; break;
				case 'Luftd.': $position[1] = $k; break;
				case 'TT': $position[2] = $k; break;
				case 'RR1': $position[3] = $k; break;
				case 'DD': $position[4] = $k; break;
				case 'FF': $position[5] = $k; break;
				case 'FX': $position[6] = $k; break;
				case 'Wetter/Wolken': $position[7] = $k; $position[8] = $k+1; break;
			}

			$k++;
		}

		preg_match($suchmuster, $wetterdaten, $treffer);
		$teile = str_word_count(trim($treffer[1]), 1, '-öäü1234567890.,');
		$teile[20] = '-';
		$zerlegen['aktuell']['hohe'] = $teile[$position[0]] . ' m';
		$zerlegen['aktuell']['luft'] = $teile[$position[1]] . ' hPa';
		$zerlegen['aktuell']['temp'] = $teile[$position[2]] . ' &deg;C';

		if ($teile[$position[2]] !== NULL)
		{
			$zerlegen['aktuell']['temp'] = $teile[$position[2]] . ' &deg;C';
		}
		else
		{
			$zerlegen['aktuell']['temp'] = '-- &deg;C';
		}

		if ($teile[$position[3]] == '----')
		{
			$teile[$position[3]] = '0.0';
		}

		$zerlegen['aktuell']['regen'] = $teile[$position[3]] . ' mm';
		$zerlegen['aktuell']['richtung'] = $teile[$position[4]] . ' ';
		$zerlegen['aktuell']['wind'] = $teile[$position[5]] . ' km/h';
		$zerlegen['aktuell']['spitze'] = $teile[$position[6]] . ' km/h';

		if ($teile[$position[7]] == 'kein')
		{
			$teile[$position[7]] = 'heiter';
		}

		$check = array('gering', 'leichter', 'starker', 'stark', 'kräftiger', 'vereinzelt', 'gefrierender', 'in', 'schweres', 'starkes');
		if (in_array($teile[$position[7]], $check))
		{
			$teile[$position[7]] = $teile[$position[7]].' '.$teile[$position[8]];
		}

		switch ($teile[$position[7]])
		{
			case 'heiter':
			case 'Heiter':
			case 'gering bewölkt':
			case 'gering Bewölkt':
				if ($stunde > 4 && $stunde < 19)
				{
					$zerlegen['aktuell']['himmel'] = 'heiter.png';
				}
				else
				{
					$zerlegen['aktuell']['himmel'] = 'nheiter.png';
				}
				break;
			case 'bewölkt':
			case 'Bewölkt':
			case 'stark bewölkt':
			case 'stark Bewölkt':
				if ($stunde > 4 && $stunde < 19)
				{
					$zerlegen['aktuell']['himmel'] = 'bewolkt.png';
				}
				else
				{
					$zerlegen['aktuell']['himmel'] = 'nbewolkt.png';
				}
				break;
			case 'bedeckt':
			case 'Bedeckt':
				$zerlegen['aktuell']['himmel'] = 'bedeckt.png'; 
				break;
			case 'Sonne':
			case 'wolkenlos':
			case 'Wolkenlos':
				if ($stunde > 4 && $stunde < 19)
				{
					$zerlegen['aktuell']['himmel'] = 'sonne.png';
				}
				else
				{
					$zerlegen['aktuell']['himmel'] = 'nsonne.png';
				}
				break;
			case 'Regenschauer':
			case 'Schauer':
			case 'leichter Regen':
			case 'leichter Schneeregen':
			case 'leichter Regen oder Schneegriesel':
			case 'Schneeregen':
			case 'Niederschlag':
				$zerlegen['aktuell']['himmel'] = 'leichtregen.png'; 
				break;
			case 'Regen':
			case 'kräftiger Regen':
			case 'kräftiger Regenschauer':
				$zerlegen['aktuell']['himmel'] = 'starkregen.png'; 
				break;
			case 'Glatteisbildung':
				$zerlegen['aktuell']['himmel'] = 'schnee.png'; 
				break;
			case 'Schnee':
			case 'Schneefall':
			case 'leichter Schneefall':
			case 'starker Schneefall':
			case 'leichter Schneeregen':
			case 'Schneeregenschauer':
			case 'kräftiger Schneefall':
			case 'kräftiger Schneeregen':
			case 'kräftiger Schneeregenschauer':
			case 'kräftiger Schneeschauer':
			case 'Schneefegen':
			case 'Schneeschauer':
			case 'Schneegriesel':
			case 'Schneetreiben':
			case 'Glatteisbildung':
				$zerlegen['aktuell']['himmel'] = 'schnee.png'; 
				break;
			case 'Graupelschauer':
			case 'Hagelschauer':
			case 'kräftiger Graupelschauer':
			case 'kräftiger Hagelschauer':
				$zerlegen['aktuell']['himmel'] = 'hagel.png'; 
				break;
			case 'Gewitter':
			case 'schweres Gewitter':
			case 'starkes Gewitter':
				$zerlegen['aktuell']['himmel'] = 'gewitter.png'; 
				break;
			case 'Nebel':
			case 'Dunst':
			case 'in Wolken':
			case 'Sandsturm':
			case 'Sandsturm oder Schneefegen':
			case 'Dunst oder flacher Nebel':
			case 'gefrierender Nebel':
				$zerlegen['aktuell']['himmel'] = 'nebel.png'; 
				break;
			default:
				if ($stunde > 4 && $stunde < 19)
				{
					$zerlegen['aktuell']['himmel'] = 'bewolkt.png';
				}
				else
				{
					$zerlegen['aktuell']['himmel'] = 'nheiter.png';
				}
				break;
		}

		$zerlegen['aktuell']['beschreibung'] = htmlentities($teile[$position[7]]);

		return $zerlegen;
	}
}
